; Driver for pocket80 "console". Provides the console routines.

; (0,X)r is the keyboard, cooked to ASCII.
; (0,0)w is an ASCII dumb tty on the display, supporting 12 to clear the screen.
; (0,1)w is raw characters to the display.
; (0,2)w is raw commands to the display.

; Keypad stuff
; Routines:	k_getraw
;					Get a raw keypress from the keypad.
;			k_conv	Index off a table pointed to by BC.
;			k_get	Convert a keypress, handling the control modes.
;			k_tab1	(table) "Normal" mode keytops
;			k_tab2	(table) "Shift" mode keytops
;			k_tab3	(table)	"Control" mode keytops
;			k_shiftmode
;			k_clearmode
;					Light or clear the LED.

k_getraw:	
			PUSH BC
k_gr1:		IN A,(0h)		; Wait for the first key
			AND 00011111b
			JR Z,k_gr1
			CALL keyblink	; "Debounce."
			LD B,0h
k_gr2:		IN A,(0h)		; Now start reading.
			AND 00011111b
			JR Z,k_gr3
			OR B
			LD B,A
			JR k_gr2
k_gr3:		CALL keyblink
			XOR A			; clear the line counter
			LD (v_ycoord),A
			LD A,B
			POP BC
			RET
     
k_conv:		PUSH HL
			LD L,A
			LD H,0h
			ADD HL,BC
			LD A,(HL)
			POP HL
			RET

k_get:		PUSH BC
			CALL k_clearmode
			CALL k_getraw
			LD BC,k_tab1
			CALL k_conv
k_get3:		CP 01h
		JR Z,k_get1
		CP 02h
		JR Z,k_get2
		POP BC
		CP 13
		CALL Z,setff
		RET
					; Valid code - normal mode.
k_get1:						; shift mode
		CALL k_shiftmode
		CALL k_getraw
		CALL k_clearmode
		LD BC,k_tab2
		CALL k_conv
		JR k_get3		; It might be another "mode" key.	
k_get2:						; control mode
		CALL k_shiftmode
		CALL k_getraw
		CALL k_clearmode
		LD BC,k_tab3
		CALL k_conv
		JR k_get3		; It might be another "mode" key.	

setff:		PUSH AF
		LD A,0FFh
		LD (v_ycoord),A
		POP AF
		RET

k_shiftmode:				; Light the "shift" LED.
		PUSH AF
		LD A,11000000b
		OUT (0h),A
		POP AF
		RET

k_clearmode:			; Clear the "shift" LED.
		PUSH AF
		LD A,01000000b
		OUT (0h),A
		POP AF
		RET

; Keymap tables.

; normal
k_tab1:		DB "@uo",8,"idl",13,"ernsjbtka cgympw",1,"vfh",2,"xzq"
;				^ zero is not a valid rawkey value! (you can't have a keypress with no keys pressed!)
;			1 and 2 are magic values for shift and function respectively
; shift
k_tab2:		DB "@UO",8,"IDL",13,"ERNSJBTKA",9,"CGYMPW",3,"VFH",4,"XZQ"
; 											^ shift-space is tab
;									                   ^ shift-shift gives 3 (cursor-mode)
;															   ^ control-shift gives 4 (function)
; function
k_tab3:		DB "@1234567890/!%*#.:'>,;+)",1,"<?",34,2,"(=-"

; LCD display stuff, loosely based on lcd.i.
; Routines: l_init  set up the LCD
;           l_cmd   output a command
;           l_char  output a character
;           l_out   output a nibble (internal)
;
; This requires general.i (for g_blink).

l_init:
        LD A,00110011b  ; Set 8 bits (twice)
        CALL l_cmd
        LD A,0010b      ; Set 4 bits
        CALL l_out
        LD A,00101000b  ; 4 bits, 1/16th duty, 5x7
        CALL l_cmd
        LD A,00000110b  ; Inc adr upon data
        CALL l_cmd
        LD A,00001110b  ; Turn display & cursor on
        CALL l_cmd
        CALL l_cls
        RET

        ; This assumes the command is 8 bits in A.
        ; Port 0 - 0-3=d4-7, 4=RS, 5=E. 6-7 should be kept LOW! RS: 0=inst, 1=char. E _/~\_
l_cmd: 	PUSH DE
        LD D,A
        AND 0F0h         ; High bits first
        rrca
        rrca
        rrca
        rrca
        CALL l_out
        LD A,D
        AND 0Fh
        CALL l_out
        LD A,D
        POP DE
        RET

l_char:
        PUSH DE
        LD D,A
        AND 0F0h         ; High bits first
        rrca
        rrca
        rrca
        rrca
        OR 10h
        CALL l_out
        LD A,D
        AND 0Fh
        OR 10h
        CALL l_out
        LD A,D
        POP DE
        RET

        ; This outputs A to port 0h, pulsing E (bit 5, $20).
l_out:  OR 01000000b		; serial output mask
		OUT (0h),A
		CALL keyblink
        OR 20h
        OUT (0h),A
		CALL keyblink
        AND 11011111b
        OUT (0h),A
		CALL keyblink
        RET

l_cls:  PUSH AF
        LD A,01h
        CALL l_cmd
		CALL keyblink
		LD A,11000000b	; line 2 locate
        CALL l_cmd
		XOR A			; clear X pos
		LD (v_xcoord),A
        POP AF
        RET

; Do a CR.
dolcdcr:
		PUSH AF
		PUSH HL
		PUSH BC
		LD A,(v_ycoord)
		INC A
		CP 2
		JR NZ,dolcdcr2
		CALL k_get		; wait if two lines have been printed without a key
		XOR A
dolcdcr2: 
		LD (v_ycoord),A
		CALL l_cls		; clear the screen.
		LD A,80h		; locate top left
		CALL l_cmd
		LD C,20
		LD HL,lcdbuf
lclrlp:	LD A,(HL)		; write out to top line...
		CALL l_char
		LD A,20h		; and clear the buffer.
		LD (HL),A
		INC HL
		DEC C
		JR NZ,lclrlp
		XOR A
		LD (v_xcoord),A
		LD A,0c0h		; locate on second line.
		CALL l_cmd
		POP BC
		POP HL
        POP AF
		RET

dobackspace:
		PUSH AF
		PUSH BC
		LD A,(v_xcoord)
		OR A
		RET Z		; can't delete back over the end of a line yet!
		DEC A
		LD (v_xcoord),A
		LD B,0h
		LD C,A
		LD HL,lcdbuf
		ADD HL,BC
		LD A,20h
		LD (HL),A
		LD A,00010000b
		CALL l_cmd ; "shift left"
		LD A,20h
		CALL l_char
		LD A,00010000b
		CALL l_cmd ; "shift left"
		POP BC
		POP AF
		RET

keyblink:
        PUSH DE
        LD E,0FFh
kgbloop:
		DEC E
        JR NZ,kgbloop
        POP DE
		RET

pocketconsoleinit: 
		LD A,01000000b
		OUT (0h),A
		LD BC,8000h
pcinitloop:
		DEC BC
		LD A,B
		OR C
		JR NZ,pcinitloop
		LD A,01000000b
		OUT (0h),A
		XOR A
		LD (v_ycoord),A		; used to track number of CRs done since last key.
		CALL l_init
		RET

pocketconsolein: JP k_get

pocketconsoleout:
		LD A,(unit)			; Check the unit number. 0=cooked, 1=raw char, 2=raw cmd.
		OR A
		JR NZ,notunit0
		LD A,C
		CP 12		; cls
		JP Z,l_cls
		CP 10		; line feed, ignore
		RET Z
		CP 13		; CR, shift display up and home cursor, and clear bottom line.
		JP Z,dolcdcr
		CP 8		; backspace
		JP Z,dobackspace
		PUSH IX
		LD IX,lcdbuf
		LD B,0h
		PUSH AF
		LD A,(v_xcoord)
		LD C,A
		POP AF
		ADD IX,BC
		PUSH AF
		LD (IX+0),A
		CALL l_char
		LD A,(v_xcoord)
		INC A
		LD (v_xcoord),A
		CP 20
		CALL Z,dolcdcr
		POP AF
		POP IX
		RET
notunit0:
		CP 2h
		JR NZ,unit2
		LD A,C
		JP l_char
unit2:	LD A,C
		JP l_cmd

pocketconsolest: 		
		IN A,(0h)
		AND 00011111b
		RET Z
		LD A,0FFh
		RET

; And, at last...
ttyinit:	EQU pocketconsoleinit
ttyout:		EQU pocketconsoleout
ttyin:		EQU pocketconsolein
ttyst:		EQU pocketconsolest

